/******************************************************************************
 * common.h
 * This file is part of "Assignment 2: Memory Management" 
 * Copyright (C) 2009 - c506001 (email: c506001@cse.hcmut.edu.vn )
 *
 * Content:
 * -------- 
 * Define common data type and functions
 *
 * System and Networking Department
 * Faculty of Computer Science and Engineering
 * Ho Chi Minh City University of Technology
 ******************************************************************************/

#ifndef _COMMON_H_
#define _COMMON_H_

#define N_SECTION_BITS  2         // page table level 1
#define N_PAGE_BITS     2         // page table level 2
#define N_OFFSET_BITS   2         // offset bits

#define START  0
#define ACCESS 1
#define FINISH 2

#define SUCCESS                   ( 0)

#define ERR_SYSTEM_NULL           (-1)
#define ERR_REQLIST_NULL          (-2)
#define ERR_FRAMELIST_NULL        (-3)
#define ERR_FREEFRAMELIST_NULL    (-4)
#define ERR_FREEFRAMELIST_EMPTY   (-5)

#define ERR_PROCLIST_NULL         (-6)
#define ERR_PROCLIST_EMPTY        (-7)
#define ERR_PROCESS_EXISTS        (-8)
#define ERR_PROCESS_NOTEXIST      (-9)
#define ERR_CREATEPROCESS_FAILED  (-10)
#define ERR_GETPROCESS_FAILED     (-11)
#define ERR_ADDPROCESS_FAILED     (-12)

#define ERR_SYSFILE_FORMAT        (-13)
#define ERR_FILEOPEN_FAILED       (-14)
#define ERR_PARSEFILE_FAILED      (-15)

#define ERR_OUT_OF_RANGE          (-16)
#define ERR_MALLOC_FAILED         (-17)
#define ERR_INVALID_ADDRESS       (-18)

#define ERR_INVALID_COMMAND       (-19)

#define TRUE 1
#define FALSE 0

typedef int bool;

struct SectionTable;

/**
 * Struct name: Request
 * Description:
 *    Contain information of a request
 **/
struct Request {
  struct Request *next;
  int cmd;        // START|ACCESS|FINISH
  int pid;        // process id
  int value;      // argument of command
};

/**
 * Struct name: RequestList
 * Description:
 *    Manage a list of requests
 **/
struct RequestList {
  struct Request *list;
  struct Request *tail;
};


/**
 * Struct name: Page
 * Description:
 *    Contain information of a page
 **/
struct Page {
  int frame;    // frame corresponding to this page
  bool valid;   // this page is valid or not
};


/**
 * Struct name: PageTable
 * Description:
 *    A page table level 2
 **/
struct PageTable {
  struct Page elems[1<<N_PAGE_BITS];
};


typedef struct PageTable* Section;
/**
 * Struct name: SectionTable
 * Description:
 *    A page table level 1
 **/
struct SectionTable {
  Section elems[1<<N_SECTION_BITS];
};


/**
 * Struct name: Frame
 * Description:
 *    Contain information of a frame
 **/
struct Frame{
  int pid;      // pid of process referring to this frame
  int page;     // no. of page that refers to this frame
  bool ref;     // reference bit
  bool inuse;   // this frame is inuse or not
};


/**
 * Struct name: FrameList
 * Description:
 *     This list point to frame table
 **/
struct FrameList{
	struct Frame *list;
	int next;       // index of the next victim frame in frame table
	int nFrames;    // number of frames in frame table
};


/**
 * Struct name: FreeFrame
 * Description:
 *    Contain information of a free frame
 **/
struct FreeFrame{
  struct FreeFrame *next;
  int no;        // no. of this frame in frame table
};


/**
 * Struct name: FreeFrameList
 * Description:
 *    Manage a list of free frames
 *    A new free frame is put at tail of this list
 **/
struct FreeFrameList{
  struct FreeFrame *list;
  struct FreeFrame *tail;
  int nFrames;    // number of free frames in the list
};


/**
 * Struct name: Request
 * Description:
 *    PCB of a process
 **/
struct Process {
  struct Process *next;
  int pid;                    // process id
  int size;                   // memory size that this process requires
  struct SectionTable stbl;   // page table of this process
};


/**
 * Struct name: ProcessList
 * Description:
 *    Manage a list of PCBs
 *    This list is arranged in the order of pid
 **/
struct ProcessList{
  struct Process *list;
  struct Process *tail;
};


/**
 * Struct name: System
 * Description:
 *    Manage system information
 **/
struct System {
  struct FrameList *frames;
  struct FreeFrameList *freeframes;
  struct ProcessList *procs;
};


// data structure-related functions
int freeFrameList(struct FrameList *frames);
int freeFreeFrameList(struct FreeFrameList *freeframes);
int freeProcess(struct Process* p);
int freeProcessList(struct ProcessList *procs);
int freeRequestList(struct RequestList *reqs);
int freeSystem(struct System *s);

int addTailFreeFrameList(struct FreeFrameList *freeframes, struct FreeFrame *ff);
int getFreeFrame(struct FreeFrameList *freeframes);
int addToProcessList(struct ProcessList *procs, struct Process *p);
int delProcessOutOfList(struct ProcessList *procs, struct Process *p);
int addTailRequestList(struct RequestList *reqs, struct Request *r);
struct Request* removeFirstRequest(struct RequestList *reqs);

#endif
