/******************************************************************************
 * memsim.c
 * This file is part of "Assignment 2: Memory Management" 
 * Copyright (C) 2009 - c506001 (email: c506001@cse.hcmut.edu.vn )
 *
 * Content:
 * -------- 
 * The Main Program
 *
 * System and Networking Department
 * Faculty of Computer Science and Engineering
 * Ho Chi Minh City University of Technology
 ******************************************************************************/

#include <stdlib.h>
#include <stdio.h> 
#include <unistd.h>
#include <string.h>
#include "common.h"
#include "fileop.h"
#include "memop.h"
#include "memsim.h"

static struct Options opts = {
  //TODO: Set default variables for opts
};

/**
 * Function name: parseArguments
 * Description:
 *    Determine what options or arguments user used
 *    Command-line options:
 * 	 [-h]        : Show help information
 *	 [-s] sysfile: Choose system config file
 *	 [-i] infile : Choose input file
 *	 [-o] outfile: Choose output file
 * Return value:
 *    0: If parsing successful
 *    exit: If error
 **/
static int parseArguments(int argc, char **argv, struct Options *options)
{
  int opt;
  extern int optind;
  extern char * optarg;    // (global variable) command-line options

  while ((opt = getopt(argc, argv, "")) != EOF)
  {
    switch (opt) {

      //TODO: Process options in command line

      default:
        fprintf(stderr, "Try 'schedsim -h' for more information\n");
        exit(1);
    }
  }

  if (optind < argc) {
    fprintf(stderr, "Invalid command syntax\n");
    fprintf(stderr, "Try 'schedsim -h' for helping\n");
    exit(1);
  }

  return 0;
}


/**
 * Function name: Usage
 * Description:
 *    Show helping information when using -h option
 **/
static void Usage(){
  printf("SYNOPSIS: schedsim [-h] [-s sysfile] [-i infile] [-o outfile]\n");
  printf("	[-h]        : Show help information\n");
  printf("	[-s] sysfile: Choose system config file\n");
  printf("	[-i] infile : Choose input file\n");
  printf("	[-o] outfile: Choose output file\n");
}


//=============================================
//========= MAIN PROGRAM ======================
//=============================================
int main(int argc, char **argv)
{
  struct RequestList *reqs;
  struct System *sys;

  // parse options (arguments)
  parseArguments(argc, argv, &opts);

  // create request list from input file
  reqs=malloc(sizeof(struct RequestList));
  if (!reqs){
    fprintf(stderr,"Can not malloc list of requests\n");
    return ERR_MALLOC_FAILED;
  }
  if (parseRequestsFromFile(reqs, opts.inFile)!=SUCCESS){
    fprintf(stderr,"Can not parse input file\n");
	  freeRequestList(reqs);
    return ERR_PARSEFILE_FAILED;
  }

  // create struct System from system config file
  sys=malloc(sizeof(struct System));
  if (!sys){
    fprintf(stderr,"Can not malloc struct System\n");
	  freeRequestList(reqs);
    return ERR_MALLOC_FAILED;
  }
  if (parseSystemInfo(sys, opts.sysFile)!=SUCCESS){
    fprintf(stderr,"Can not parse system config file\n");
    freeRequestList(reqs);
    free(sys);
    return ERR_PARSEFILE_FAILED;
  }

  // simulate memory management
  printf("Starting simulation ... \n");
  while(reqs->list) {
    char *output;
    struct Request *req=removeFirstRequest(reqs);

    switch (req->cmd) {
      case START:
        allocFrames(sys,req->pid,req->value,&output);
        writeToFile(opts.outFile,output);
        break;
      case ACCESS:
        accessAddress(sys,req->pid,req->value,&output);
        writeToFile(opts.outFile,output);
        break;
      case FINISH:
        finishProcess(sys,req->pid,&output);
        writeToFile(opts.outFile,output);
        break;
      default:
        fprintf(stderr,"Error in retrieving command request\n");
        freeRequestList(reqs);
        free(sys);
        return ERR_INVALID_COMMAND;
    }
  }

  // free allocated memory
  freeRequestList(reqs);
  freeSystem(sys);

  printf("Finished\n");
  return 0;
}
